@@ -6,12 +6,16 @@ import logging
 import requests
 from datetime import date, timedelta
 
-from .obj import Agents, User, TokenScope, SearchTypes
-from .folders import Folders
-from .uploads import Uploads
-from .jobs import Jobs
-from .report import Report
-from .exceptions import AuthenticationError, FossologyApiError
+from fossology.obj import Agents, User, TokenScope, SearchTypes, get_options
+from fossology.folders import Folders
+from fossology.uploads import Uploads
+from fossology.jobs import Jobs
+from fossology.report import Report
+from fossology.exceptions import (
+    AuthenticationError,
+    AuthorizationError,
+    FossologyApiError,
+)
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
@@ -36,12 +40,12 @@ def fossology_token(
     :param password: the password of the user
     :param name: the name of the token
     :param scope: the scope of the token (default: READ)
-    :param expire: the expire date of the token (default max. 30 days)
+    :param expire: the expire date of the token (default: max. 30 days)
     :type url: string
     :type username: string
     :type password: string
     :type name: string
-    :type scope: TokenScope (default TokenScope.READ)
+    :type scope: TokenScope (default: TokenScope.READ)
     :type expire: string, e.g. 2019-12-25
     :return: the new token
     :rtype: string
@@ -189,7 +193,8 @@ class Fossology(Folders, Uploads, Jobs, Report):
                     continue
                 if user.get("email"):
                     foss_user = User.from_json(user)
-                    if agents := user.get("agents"):
+                    agents = user.get("agents")
+                    if agents:
                         foss_user.agents = Agents.from_json(agents)
                     users_list.append(foss_user)
             return users_list
@@ -207,7 +212,7 @@ class Fossology(Folders, Uploads, Jobs, Report):
         :raises FossologyApiError: if the REST call failed
         """
         response = self.session.delete(f"{self.api}/users/{user.id}")
-        print(response.json())
+
         if response.status_code == 202:
             return
         else:
@@ -223,6 +228,7 @@ class Fossology(Folders, Uploads, Jobs, Report):
         filesizemax=None,
         license=None,
         copyright=None,
+        group=None,
     ):
         """Search for a specific file
 
@@ -235,6 +241,7 @@ class Fossology(Folders, Uploads, Jobs, Report):
         :param filesizemax: Max filesize in bytes
         :param license: License search filter
         :param copyright: Copyright search filter
+        :param group: the group name to choose while performing search (default: None)
         :type searchType: SearchType Enum
         :type filename: string
         :type tag: string
@@ -242,9 +249,11 @@ class Fossology(Folders, Uploads, Jobs, Report):
         :type filesizemax: int
         :type license: string
         :type copyright: string
+        :type group: string
         :return: list of items corresponding to the search criteria
         :rtype: JSON
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group
         """
         headers = {"searchType": searchType.value}
         if filename:
@@ -259,10 +268,18 @@ class Fossology(Folders, Uploads, Jobs, Report):
             headers["license"] = license
         if copyright:
             headers["copyright"] = copyright
+        if group:
+            headers["groupName"] = group
 
         response = self.session.get(f"{self.api}/search", headers=headers)
+
         if response.status_code == 200:
             return response.json()
+
+        elif response.status_code == 403:
+            description = f"Searching {get_options(group)}not authorized"
+            raise AuthorizationError(description, response)
+
         else:
             description = "Unable to get a result with the given search criteria"
             raise FossologyApiError(description, response)
