@@ -1,12 +1,13 @@
-# Copyright 2019 Siemens AG
+# Copyright 2019-2020 Siemens AG
 # SPDX-License-Identifier: MIT
 
 import re
 import time
 import logging
 
-from tenacity import retry, TryAgain, stop_after_attempt
-from .exceptions import FossologyApiError
+from tenacity import retry, TryAgain, stop_after_attempt, retry_if_exception_type
+from fossology.exceptions import FossologyApiError, AuthorizationError
+from fossology.obj import get_options
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
@@ -15,41 +16,53 @@ logger.setLevel(logging.DEBUG)
 class Report:
     """Class dedicated to all "report" related endpoints"""
 
-    @retry(stop=stop_after_attempt(3))
-    def generate_report(self, upload, report_format=None):
+    @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(3))
+    def generate_report(self, upload, report_format=None, group=None):
         """Generate a report for a given upload
 
         API Endpoint: GET /report
 
         :param upload: the upload which report will be generated
-        :param format: the report format (default ReportFormat.READMEOSS)
+        :param format: the report format (default: ReportFormat.READMEOSS)
+        :param group: the group name to choose while generating the report (default: None)
         :type upload: Upload
         :type format: ReportFormat
+        :type group: string
         :return: the report id
         :rtype: int
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group
         """
         headers = {"uploadId": str(upload.id)}
         if report_format:
             headers["reportFormat"] = report_format.value
         else:
             headers["reportFormat"] = "readmeoss"
+        if group:
+            headers["groupName"] = group
+
+        response = self.session.get(f"{self.api}/report", headers=headers)
 
-        response = self.session.get(self.api + "/report", headers=headers)
         if response.status_code == 201:
             report_id = re.search("[0-9]*$", response.json()["message"])
             return report_id[0]
+
+        elif response.status_code == 403:
+            description = f"Generating report for upload {upload.id} {get_options(group)}not authorized"
+            raise AuthorizationError(description, response)
+
         elif response.status_code == 503:
             wait_time = response.headers["Retry-After"]
-            logger.debug(f"Report is not ready yet, retry after {wait_time} seconds")
+            logger.debug(f"Retry generate report after {wait_time} seconds")
             time.sleep(int(wait_time))
             raise TryAgain
+
         else:
             description = f"Report generation for upload {upload.name} failed"
             raise FossologyApiError(description, response)
 
-    @retry(stop=stop_after_attempt(3))
-    def download_report(self, report_id, as_zip=False):
+    @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(3))
+    def download_report(self, report_id, as_zip=False, group=None):
         """Download a report
 
         API Endpoint: GET /report/{id}
@@ -67,29 +80,36 @@ class Report:
         >>>     report_file.write(report_content)
 
         :param report_id: the id of the generated report
-        :param as_zip: control if the report should be generated as ZIP file (default False)
+        :param as_zip: control if the report should be generated as ZIP file (default: False)
+        :param group: the group name to choose while downloading a specific report (default: None)
         :type report_id: int
         :type as_zip: boolean
+        :type group: string
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group
         """
         if as_zip:
             headers = {"Accept": "application/zip"}
         else:
             headers = {"Accept": "text/plain"}
-        while True:
-            response = self.session.get(
-                self.api + f"/report/{report_id}", headers=headers
+        if group:
+            headers["groupName"] = group
+
+        response = self.session.get(f"{self.api}/report/{report_id}", headers=headers)
+        if response.status_code == 200:
+            return response.text
+
+        elif response.status_code == 403:
+            description = (
+                f"Getting report {report_id} {get_options(group)}not authorized"
             )
-            if response.status_code == 200:
-                return response.text
-            elif response.status_code == 503:
-                wait_time = response.headers["Retry-After"]
-                logger.debug(
-                    f"Report is not ready yet, retry after {wait_time} seconds"
-                )
-                time.sleep(int(wait_time))
-                raise TryAgain
-            else:
-                description = "Download of report {report_id} failed"
-                raise FossologyApiError(description, response)
-                return None
+            raise AuthorizationError(description, response)
+
+        elif response.status_code == 503:
+            wait_time = response.headers["Retry-After"]
+            logger.debug(f"Retry get report after {wait_time} seconds")
+            time.sleep(int(wait_time))
+            raise TryAgain
+        else:
+            description = "Download of report {report_id} failed"
+            raise FossologyApiError(description, response)
