@@ -6,8 +6,8 @@ import time
 import logging
 from tenacity import retry, retry_if_exception_type, stop_after_attempt, TryAgain
 
-from .obj import Upload, Summary
-from .exceptions import FossologyApiError
+from fossology.obj import Upload, Summary, get_options
+from fossology.exceptions import AuthorizationError, FossologyApiError
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
@@ -18,27 +18,41 @@ class Uploads:
 
     # Retry until the unpack agent is finished
     @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(10))
-    def detail_upload(self, upload_id):
+    def detail_upload(self, upload_id, group=None):
         """Get detailled information about an upload
 
         API Endpoint: GET /uploads/{id}
 
         :param upload_id: the id of the upload
+        :param group: the group the upload shall belong to
         :type: int
+        :type group: string
         :return: the upload data
         :rtype: Upload
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group
         """
-        response = self.session.get(f"{self.api}/uploads/{upload_id}")
+        headers = {}
+        if group:
+            headers["groupName"] = group
+        response = self.session.get(f"{self.api}/uploads/{upload_id}", headers=headers)
+
         if response.status_code == 200:
             logger.debug(f"Got details for upload {upload_id}")
             return Upload.from_json(response.json())
+
+        elif response.status_code == 403:
+            description = f"Getting details for upload {upload_id} {get_options(group)}not authorized"
+            raise AuthorizationError(description, response)
+
         elif response.status_code == 503:
+            wait_time = response.headers["Retry-After"]
             logger.debug(
-                f"Unpack agent for {upload_id} didn't start yet, is the scheduler running?"
+                f"Retry GET upload {upload_id} after {wait_time} seconds: {response.json()['message']}"
             )
-            time.sleep(20)
+            time.sleep(int(wait_time))
             raise TryAgain
+
         else:
             description = f"Error while getting details for upload {upload_id}"
             raise FossologyApiError(description, response)
@@ -54,7 +68,7 @@ class Uploads:
         ignore_scm=False,
         group=None,
     ):
-        """Upload a file to FOSSology
+        """Upload a package to FOSSology
 
         API Endpoint: POST /uploads
 
@@ -108,7 +122,7 @@ class Uploads:
         :param url: the URL specification to upload from a url
         :param description: description of the upload (default: None)
         :param access_level: access permissions of the upload (default: protected)
-        :param ignore_scm: ignore SCM files (Git, SVN, TFS) (default: False)
+        :param ignore_scm: ignore SCM files (Git, SVN, TFS) (default: True)
         :param group: the group name to chose while uploading the file (default: None)
         :type folder: Folder
         :type file: string
@@ -121,6 +135,7 @@ class Uploads:
         :return: the upload data
         :rtype: Upload
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group
         """
         headers = {"folderId": str(folder.id)}
         if description:
@@ -128,7 +143,7 @@ class Uploads:
         if access_level:
             headers["public"] = access_level.value
         if ignore_scm:
-            headers["ignoreScm"] = "true"
+            headers["ignoreScm"] = "false"
         if group:
             headers["groupName"] = group
 
@@ -139,24 +154,30 @@ class Uploads:
                 response = self.session.post(
                     f"{self.api}/uploads", files=files, headers=headers
                 )
-        elif vcs:
-            headers["uploadType"] = "vcs"
-            data = json.dumps(vcs)
-            headers["Content-Type"] = "application/json"
-            response = self.session.post(
-                f"{self.api}/uploads", data=data, headers=headers
-            )
-        elif url:
-            headers["uploadType"] = "url"
-            data = json.dumps(url)
+        elif vcs or url:
+            if vcs:
+                headers["uploadType"] = "vcs"
+                data = json.dumps(vcs)
+            else:
+                headers["uploadType"] = "url"
+                data = json.dumps(url)
             headers["Content-Type"] = "application/json"
             response = self.session.post(
                 f"{self.api}/uploads", data=data, headers=headers
             )
         else:
-            logger.debug("Neither VCS or File option given, not uploading anything")
+            logger.debug(
+                "Neither VCS, or Url or filename option given, not uploading anything"
+            )
             return
 
+        if file:
+            source = f"{file}"
+        elif vcs:
+            source = vcs.get("vcsName")
+        else:
+            source = url.get("name")
+
         if response.status_code == 201:
             try:
                 upload = self.detail_upload(response.json()["message"])
@@ -166,30 +187,48 @@ class Uploads:
                 )
                 return upload
             except TryAgain:
-                if file:
-                    source = f"{file}"
-                elif vcs:
-                    source = vcs.get("vcsName")
-                else:
-                    source = url.get("name")
                 description = f"Upload of {source} failed"
                 raise FossologyApiError(description, response)
 
+        elif response.status_code == 403:
+            description = (
+                f"Upload of {source} {get_options(group, folder)}not authorized"
+            )
+            raise AuthorizationError(description, response)
+
+        else:
+            description = f"Upload {description} could not be performed"
+            raise FossologyApiError(description, response)
+
     @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(3))
-    def upload_summary(self, upload):
+    def upload_summary(self, upload, group=None):
         """Get clearing information about an upload
 
         API Endpoint: GET /uploads/{id}/summary
 
         :param upload: the upload to gather data from
+        :param group: the group name to chose while accessing an upload (default: None)
         :type: Upload
+        :type group: string
         :return: the upload summary data
         :rtype: Summary
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group
         """
-        response = self.session.get(f"{self.api}/uploads/{upload.id}/summary")
+        headers = {}
+        if group:
+            headers["groupName"] = group
+        response = self.session.get(
+            f"{self.api}/uploads/{upload.id}/summary", headers=headers
+        )
+
         if response.status_code == 200:
             return Summary.from_json(response.json())
+
+        elif response.status_code == 403:
+            description = f"Getting summary of upload {upload.id} {get_options(group)}not authorized"
+            raise AuthorizationError(description, response)
+
         elif response.status_code == 503:
             logger.debug(
                 f"Unpack agent for {upload.uploadname} (id={upload.id}) didn't start yet"
@@ -201,49 +240,59 @@ class Uploads:
             raise FossologyApiError(description, response)
 
     @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(3))
-    def upload_licenses(self, upload, agent=None, containers=False):
+    def upload_licenses(self, upload, agent=None, containers=False, group=None):
         """Get clearing information about an upload
 
         API Endpoint: GET /uploads/{id}/licenses
 
         :param upload: the upload to gather data from
-        :param agent: the license agent to use (default: LicenseAgent.MONK)
+        :param agent: the license agents to use (e.g. "nomos,monk,ninka,ojo,reportImport", default: "nomos")
         :param containers: wether to show containers or not (default: False)
+        :param group: the group name to chose while accessing the upload (default: None)
         :type upload: Upload
-        :type agent: LicenseAgent
+        :type agent: string
         :type containers: boolean
+        :type group: string
         :return: the licenses found by the specified agent
         :rtype: list of licenses as JSON object
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group
         """
+        params = {}
+        headers = {}
+        if group:
+            headers["groupName"] = group
         if agent:
-            agent = agent.value
+            params["agent"] = agent
         else:
-            agent = "nomos"
+            params["agent"] = agent = "nomos"
         if containers:
-            containers = "true"
-            response = self.session.get(
-                f"{self.api}/uploads/{upload.id}/licenses?agent={agent}&containers={containers}"
-            )
-        else:
-            response = self.session.get(
-                f"{self.api}/uploads/{upload.id}/licenses?agent={agent}"
-            )
+            params["containers"] = "true"
+        response = self.session.get(
+            f"{self.api}/uploads/{upload.id}/licenses", headers=headers, params=params
+        )
 
         if response.status_code == 200:
             return response.json()
+
+        elif response.status_code == 403:
+            description = f"Getting license for upload {upload.id} {get_options(group)}not authorized"
+            raise AuthorizationError(description, response)
+
         elif response.status_code == 412:
             logger.info(
-                f"Agent {agent} has not been scheduled for {upload.uploadname} (id={upload.id}): "
+                f"Unable to get licenses from {agent} for {upload.uploadname} (id={upload.id}): "
                 f"{response.json()['message']}"
             )
             return
+
         elif response.status_code == 503:
             logger.debug(
                 f"Unpack agent for {upload.uploadname} (id={upload.id}) didn't start yet"
             )
             time.sleep(3)
             raise TryAgain
+
         else:
             description = f"No licenses for upload {upload.uploadname} (id={upload.id})"
             raise FossologyApiError(description, response)
@@ -254,10 +303,11 @@ class Uploads:
         API Endpoint: DELETE /uploads/{id}
 
         :param upload: the upload to be deleted
-        :param group: the group name to chose while deleting the upload (default None)
+        :param group: the group name to chose while deleting the upload (default: None)
         :type upload: Upload
         :type group: string
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group
         """
         headers = {}
         if group:
@@ -265,27 +315,70 @@ class Uploads:
         response = self.session.delete(
             f"{self.api}/uploads/{upload.id}", headers=headers
         )
+
         if response.status_code == 202:
             logger.info(f"Upload {upload.id} has been scheduled for deletion")
+
+        elif response.status_code == 403:
+            description = (
+                f"Deleting upload {upload.id} {get_options(group)}not authorized"
+            )
+            raise AuthorizationError(description, response)
+
         else:
             description = f"Unable to delete upload {upload.id}"
             raise FossologyApiError(description, response)
 
-    def list_uploads(self):
+    def list_uploads(
+        self, folder=None, group=None, recursive=True, page_size=20, page=1
+    ):
         """Get all uploads available to the registered user
 
         API Endpoint: GET /uploads
 
+        :param folder: only list uploads from the given folder
+        :param group: list uploads from a specific group (not only your own uploads) (default: None)
+        :param recursive: wether to list uploads from children folders or not (default: True)
+        :param page_size: limit the number of uploads per page (default: 20)
+        :param page: the number of the page to fetch uploads from (default: 1)
+        :type folder: Folder
+        :type group: string
+        :type recursive: boolean
+        :type page_size: int
+        :type page: int
         :return: a list of uploads
         :rtype: list of Upload
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group
         """
-        response = self.session.get(f"{self.api}/uploads")
+        params = {}
+        headers = {"limit": str(page_size), "page": str(page)}
+        if group:
+            headers["groupName"] = group
+        if folder:
+            params["folderId"] = folder.id
+        if not recursive:
+            params["recursive"] = "false"
+
+        response = self.session.get(
+            f"{self.api}/uploads", headers=headers, params=params
+        )
+
         if response.status_code == 200:
             uploads_list = list()
             for upload in response.json():
                 uploads_list.append(Upload.from_json(upload))
+            logger.info(
+                f"Retrieved page {page} of uploads, {response.headers['X-TOTAL-PAGES']} pages are in total available"
+            )
             return uploads_list
+
+        elif response.status_code == 403:
+            description = (
+                f"Retrieving list of uploads {get_options(group, folder)}not authorized"
+            )
+            raise AuthorizationError(description, response)
+
         else:
             description = "Unable to retrieve the list of uploads"
             raise FossologyApiError(description, response)
@@ -297,11 +390,12 @@ class Uploads:
 
         :param upload: the Upload to be copied in another folder
         :param folder: the destination Folder
-        :param group: the group name to chose while deleting the upload (default None)
+        :param group: the group name to chose while changing the upload (default: None)
         :type upload: Upload
         :type folder: Folder
         :type group: string
         :raises FossologyApiError: if the REST call failed
+        :raises AuthorizationError: if the user can't access the group or folder
         """
         headers = {"folderId": str(folder.id)}
         if group:
@@ -309,8 +403,16 @@ class Uploads:
         response = self.session.patch(
             f"{self.api}/uploads/{upload.id}", headers=headers
         )
+
         if response.status_code == 202:
             logger.info(f"Upload {upload.uploadname} has been moved to {folder.name}")
+
+        elif response.status_code == 403:
+            description = (
+                f"Moving upload {upload.id} {get_options(group, folder)}not authorized"
+            )
+            raise AuthorizationError(description, response)
+
         else:
             description = f"Unable to move upload {upload.uploadname} to {folder.name}"
             raise FossologyApiError(description, response)
@@ -328,8 +430,14 @@ class Uploads:
         """
         headers = {"folderId": str(folder.id)}
         response = self.session.put(f"{self.api}/uploads/{upload.id}", headers=headers)
+
         if response.status_code == 202:
             logger.info(f"Upload {upload.uploadname} has been copied to {folder.name}")
+
+        elif response.status_code == 403:
+            description = f"Copy upload {upload.id} {get_options(folder)}not authorized"
+            raise AuthorizationError(description, response)
+
         else:
             description = f"Unable to copy upload {upload.uploadname} to {folder.name}"
             raise FossologyApiError(description, response)
